/**
 * Conway's Game of Life 
 * by Mike Davis. 
 * 
 * This program is a simple version of Conway's 
 * game of Life.  A lit point turns off if there 
 * are fewer than two or more than three surrounding 
 * lit points.  An unlit point turns on if there 
 * are exactly three lit neighbors.  The 'density' 
 * parameter determines how much of the board will 
 * start out lit.  
 */
import processing.serial.*;
Serial myPort;      // The serial port 
int sx, sy; 
float density = 0.5; 
int[][][] world;
int[] aPixels;
int resetcounter=0;
PImage seed;
boolean autoreset = true;
void setup() 
{ 
  String portName = Serial.list()[1];
  myPort = new Serial(this, portName, 57600);
  aPixels = new int[width*height];
  size(32, 32);
  frameRate(25);
  sx = width;
  sy = height;
  world = new int[sx][sy][2]; 
  seed = loadImage("gliders.bmp");
  
  stroke(255); 
   
  pictureSeed();
  
  for(int x=0; x<3; x++) {myPort.write(0x00);} //dummy writes after reset
  int s = second();
  while ((second()-s)<3){};  
} 
 
void draw() 
{ 
  background(0); 
 resetcounter++;
  if((resetcounter>500) && autoreset) 
  {
    Seed();
    resetcounter=0;
  }
  for(int i=0; i<width*height; i++) {aPixels[i]=0;}
  // Drawing and update cycle 
  for (int x = 0; x < sx; x=x+1) { 
    for (int y = 0; y < sy; y=y+1) { 
      if ((world[x][y][1] == 1) || (world[x][y][1] == 0 && world[x][y][0] == 1)) 
      { 
        world[x][y][0] = 1; 
        point(x, y);
        aPixels[x+(y*32)]=100; 
      } 
      if (world[x][y][1] == -1) 
      { 
        world[x][y][0] = 0; 
      } 
      world[x][y][1] = 0; 
    } 
  } 
  // Birth and death cycle 
  for (int x = 0; x < sx; x=x+1) { 
    for (int y = 0; y < sy; y=y+1) { 
      int count = neighbors(x, y); 
      if (count == 3 && world[x][y][0] == 0) 
      { 
        world[x][y][1] = 1; 
      } 
      if ((count < 2 || count > 3) && world[x][y][0] == 1) 
     { 
        world[x][y][1] = -1; 
      } 
    } 
  } 
ledDisplayWrite();
} 
 
// Count the number of adjacent cells 'on' 
int neighbors(int x, int y) 
{ 
  return world[(x + 1) % sx][y][0] + 
         world[x][(y + 1) % sy][0] + 
         world[(x + sx - 1) % sx][y][0] + 
         world[x][(y + sy - 1) % sy][0] + 
         world[(x + 1) % sx][(y + 1) % sy][0] + 
         world[(x + sx - 1) % sx][(y + 1) % sy][0] + 
         world[(x + sx - 1) % sx][(y + sy - 1) % sy][0] + 
         world[(x + 1) % sx][(y + sy - 1) % sy][0]; 
} 

void keyPressed()
{
  pictureSeed();
  //  for (int i = 0; i < sx * sy * density; i++) { 
 //   world[(int)random(sx)][(int)random(sy)][1] = 1; 
 // } 
}

void ledDisplayWrite()
{
        // This function converts a Pixel class image to a 128 byte 
        // variable which can be send to the LED display
        
        byte crunch[];
        crunch = new byte [129];
        for (int x=0; x<128; x++){crunch[x]=0;}
        for(int x=0; x<128; x++) {
        for (int c=0;c<8;c++){
         if((int)brightness(aPixels[(x*8)+c])>0) crunch[x] += (byte)pow(2,c);
        }
       }
  myPort.write(crunch);
}

void pictureSeed()
{
    for (int i =0;i < 32;i++) {
    for (int j =0;j < 32;j++) {
      color pix = seed.get(i, j);
      if (red(pix)>100) {
        world[i][j][0] = 0;
        world[i][j][1] = 0;
      } else {
        world[i][j][0] = 1;
        world[i][j][1] = 0;
      }
    }
  }
}

void Seed()
{  // Set random cells to 'on' 
  for (int i = 0; i < sx * sy * density; i++) { 
    world[(int)random(sx)][(int)random(sy)][1] = 1; 
  } 
}

